"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const base_1 = require("../base");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const message_1 = require("../services/message");
const track_1 = require("../services/track");
const util_1 = require("../util");
const { resolve } = path;
class RunBase extends base_1.default {
    constructor() {
        super(...arguments);
        this.cmd = '';
        this.watch = false;
    }
    async preInit() {
        // @ts-ignore
        const { flags, args } = this.parse(this.constructor);
        this.cmd = args.cmd;
        this.watch = flags.watch;
        this.extras = flags.extras;
        const cmd = this.cmd || this.constructor.id;
        track_1.uploadHubble({ cmd });
        return flags;
    }
    async initialize() {
        await util_1.default.asyncExec(`rm -rf ${conf_1.default.tmpDir} && mkdir ${conf_1.default.tmpDir}`);
        const { appConfPath, rdcConfPath, cwd, RdTypes } = conf_1.default;
        if (fs.existsSync(appConfPath)) {
            conf_1.default.rdType = RdTypes.Application;
        }
        else if (fs.existsSync(rdcConfPath)) {
            conf_1.default.rdType = RdTypes.Container;
        }
        logger_1.debug(`rd type recognized as ${conf_1.default.rdType}`);
        if (!conf_1.default.rdType) {
            throw Error('no rde config file found, please read docs first');
        }
        const appDir = resolve(cwd, 'app');
        const templateDir = resolve(cwd, conf_1.default.templateDir);
        if ((conf_1.default.rdType === RdTypes.Container && !fs.existsSync(templateDir)) ||
            !fs.existsSync(appDir)) {
            throw Error(message_1.MCOMMON.WRONG_PROJECT_STRUCTURE);
        }
    }
    async run() { }
}
RunBase.args = [{
        name: 'cmd',
        required: false,
        description: 'scripts provided by container',
    }];
RunBase.flags = Object.assign({}, base_1.default.flags, { watch: command_1.flags.boolean({ char: 'w' }), extras: command_1.flags.string({
        char: 'e',
        description: 'arguments need to pass to npm run cmd',
    }) });
tslib_1.__decorate([
    track_1.TError({ conf: conf_1.default }),
    track_1.TStart
], RunBase.prototype, "initialize", null);
exports.default = RunBase;
