"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const extend = require("deep-extend");
const fs = require("fs");
const path = require("path");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const render_1 = require("../services/render");
const sync_1 = require("../services/sync");
const watcher_1 = require("../services/watcher");
const util_1 = require("../util");
const { resolve } = path;
const { templateDir, integrateDir, runtimeDir, dockerWorkDirRoot, RdTypes, } = conf_1.default;
class DockerRun {
    constructor({ watch }) {
        this.watch = watch;
    }
    async start() {
        // merge rda.config rdc config
        this.config = await this.mergeRdcConf();
        if (conf_1.default.isIntegrate) {
            // template render to .integrated
            await this.renderDir(templateDir, integrateDir);
            // compose rde and watch app
            await this.composeRde();
        }
        else {
            // template render to runtime
            await this.renderDir(templateDir, runtimeDir);
        }
        if (conf_1.default.rdType === RdTypes.Application) {
            await sync_1.default.mergePkgJson(resolve(dockerWorkDirRoot, 'app', 'package.json'), resolve(dockerWorkDirRoot, conf_1.default.rdeDir, 'package.json'), resolve(dockerWorkDirRoot, conf_1.default.rdeDir));
        }
        if (this.watch && conf_1.default.rdType === RdTypes.Container) {
            await this.watchTemplate(conf_1.default.rdeDir);
        }
    }
    async mergeRdcConf() {
        const { cwd, appConfName, rdcConfName, } = conf_1.default;
        let config = require(resolve(cwd, rdcConfName));
        const rdaConfPath = resolve(cwd, appConfName);
        if (fs.existsSync(rdaConfPath)) {
            config = extend(config, require(rdaConfPath));
        }
        conf_1.default.rdMode = config.mode || conf_1.default.RdModes.Integrate;
        return config;
    }
    async renderDir(from, to) {
        // @ts-ignore
        const { render: rdtRender = {}, container, suites } = this.config;
        const { includes = [] } = rdtRender;
        if (container && rdtRender.validate) {
            const result = rdtRender.validate(container.render);
            if (result !== true) {
                throw Error(result);
            }
        }
        if (suites && suites.length) {
            container.render = container.render || {};
            Object.keys(conf_1.default.frameworks).forEach(framework => {
                container.render[`${framework}Suites`] = suites.filter(item => {
                    if (item.framework === framework) {
                        const arr = item.split('/');
                        let alias = item;
                        if (arr.length > 1) {
                            alias = arr[1];
                        }
                        alias = alias.replace(/[-_]/gim, '').toUpperCase();
                        return {
                            name: item,
                            alias,
                        };
                    }
                    return false;
                });
            });
        }
        const dataView = container ? container.render : rdtRender.mock || {};
        const options = {
            overwrite: true,
            dot: true,
        };
        if (conf_1.default.isIntegrate) {
            options.filter = /.*(?<!\.d\.ts)$/;
        }
        await render_1.default.renderDir(from, Object.assign({}, dataView), includes, to, options);
    }
    async composeRde() {
        logger_1.debug(`merged mappings: ${JSON.stringify(this.config.mappings)}`);
        for (let { from, to, options } of this.config.mappings) {
            const appDir = resolve(conf_1.default.cwd, from);
            const destDir = resolve(integrateDir, to);
            if (!options || options.filter) {
                options = options || {};
                options.filter = /.*(?<!\.d\.ts)$/;
            }
            await util_1.default.copy(appDir, destDir, { options });
        }
        if (this.watch) {
            let { mappings } = this.config;
            new watcher_1.default(mappings).start();
        }
    }
    async watchTemplate(dest) {
        const mappings = [{
                from: templateDir,
                to: dest
            }];
        new watcher_1.default(mappings, this.renderDir.bind(this)).start();
    }
}
exports.default = DockerRun;
