"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Table = require("cli-table2");
const fs = require("fs-extra");
const path = require("path");
const semver = require("semver");
const conf_1 = require("../services/conf");
const npm_1 = require("../services/npm");
const util_1 = require("../util");
async function default_1({ config }) {
    const file = path.join(config.cacheDir, 'version');
    const delayInDays = 6;
    const needUpdate = async () => {
        try {
            const { mtime } = await fs.stat(file);
            const expireDate = new Date(mtime.valueOf() + 1000 * 60 * 60 * 24 * delayInDays);
            return expireDate < new Date();
        }
        catch (e) {
            if (e.code !== 'ENOENT')
                throw e;
            return true;
        }
    };
    const update = async () => {
        const { latest } = await fs.readJson(file);
        if (semver.gt(latest, config.version)) {
            const table = new Table({
                head: ['package', 'current', 'latest'],
                style: {
                    'padding-left': 1,
                    'padding-right': 1,
                    head: ['cyan'],
                    border: ['white']
                },
                colWidths: [15, 25, 25]
            });
            table.push([config.name, config.version, latest]);
            table.push([
                {
                    content: 'New version available. Updating RDE Cli Automatically',
                    colSpan: 3
                }
            ]);
            // tslint:disable:no-console
            console.log(table.toString());
            await npm_1.default.install({
                pkgs: [config.name],
                isGlobal: true,
                isDevDep: false,
            });
        }
    };
    const updateVersion = async () => {
        const pkgInfo = await npm_1.default.getInfo(config.name);
        await fs.outputFile(file, JSON.stringify(Object.assign({}, pkgInfo['dist-tags'], { current: config.version })));
        await update();
    };
    if (await needUpdate()) {
        await updateVersion();
    }
    await util_1.default.copy(file, path.join(conf_1.default.localCacheDir, 'version'), {});
}
exports.default = default_1;
