"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const hl = require("highlight.js");
const MarkdownIt = require("markdown-it");
const markdownMeta = require("markdown-it-meta");
const api_1 = require("./api");
const component_1 = require("./component");
const table_1 = require("./table");
const mdIt = new MarkdownIt({
    highlight(str, lang) {
        if (lang && hl.getLanguage(lang)) {
            try {
                return '<pre class="hljs"><code>' +
                    hl.highlight(lang, str, true).value +
                    '</code></pre>';
            }
            catch (e) {
                if (e) { }
            }
        }
        return `<pre class="hljs"><code>${mdIt.utils.escapeHtml(str)}</code></pre>`;
    }
});
mdIt.renderer.rules.heading_open = (tokens, idx) => {
    const token = tokens[idx];
    const { tag } = token;
    return `<${tag} class="rde-${tag}">`;
};
mdIt.renderer.rules.paragraph_open = (tokens, idx) => {
    const token = tokens[idx];
    const { tag } = token;
    return `<${tag} class="rde-${tag}">`;
};
mdIt.renderer.rules.link_open = (tokens, idx, _options, _env, slf) => {
    const token = tokens[idx];
    return `<a class="rde-a"${slf.renderAttrs(token)}>`;
};
mdIt.use(markdownMeta);
mdIt.use(api_1.default);
mdIt.use(component_1.default);
mdIt.use(table_1.default);
exports.default = mdIt;
